@@ -16,7 +16,7 @@ use crate::proc::{self, ContainerInfo, ProcTable, Process, ProcessKey};
 #[cfg(all(feature = "procfs", target_os = "linux"))]
 use crate::procfs;
 #[cfg(target_os = "linux")]
-use crate::sockaddr::SocketAddr;
+use crate::sockaddr::{SocketAddr, SocketAddrMatcher};
 use crate::types::*;
 use crate::userdb::UserDB;
 
@@ -90,6 +90,7 @@ pub struct Settings {
     pub filter_keys: HashSet<Vec<u8>>,
     pub filter_labels: HashSet<Vec<u8>>,
     pub filter_null_keys: bool,
+    pub filter_sockaddr: Vec<SocketAddrMatcher>,
     pub filter_raw_lines: regex::bytes::RegexSet,
     pub filter_first_per_process: bool,
 }
@@ -124,6 +125,7 @@ impl Default for Settings {
             filter_keys: HashSet::new(),
             filter_labels: HashSet::new(),
             filter_null_keys: false,
+            filter_sockaddr: vec![],
             filter_raw_lines: regex::bytes::RegexSet::empty(),
             filter_first_per_process: false,
         }
@@ -279,7 +281,6 @@ impl UserGroupIDs {
 
 /// Returns a script name from path if exe's dev / inode don't match
 ///
-/// This seems to work with Docker containers but not with Podman.
 /// The executable's device and inode are inspected throguh the
 /// /proc/<pid>/root/ symlink. This may fail for
 ///
@@ -288,10 +289,16 @@ impl UserGroupIDs {
 ///   using fuse-overlayfs (observed with
 ///   podman+fuse-overlayfs/1.4.0-1 on Debian/buster).
 ///
+/// Some container setups construct filesystem mappings where
+/// major(dev) = 0: "Unnamed devices (e.g. non-device mounts)". In
+/// this case, no script is returned if exe is found on a device with
+/// major(dev) != 0.
+///
 /// As an extra sanity check, exe is compared with normalized
 /// PATH.name. If they are equal, no script is returned.
 #[cfg(all(feature = "procfs", target_os = "linux"))]
 fn path_script_name(path: &Body, pid: u32, ppid: u32, cwd: &[u8], exe: &[u8]) -> Option<NVec> {
+    use nix::sys::stat::{major, makedev};
     use std::{
         ffi::OsStr,
         os::unix::{ffi::OsStrExt, fs::MetadataExt},
@@ -306,10 +313,19 @@ fn path_script_name(path: &Body, pid: u32, ppid: u32, cwd: &[u8], exe: &[u8]) ->
 
     let mut p_dev: Option<u64> = None;
     let mut p_inode: Option<u64> = None;
-    let mut name = None;
+    let mut p_name = None;
     for (k, v) in path {
+        if k == "item" && *v != Value::Number(Number::Dec(0)) {
+            // Can't determine a script if the first PATH record is
+            // missing from the audit log.
+            break;
+        }
         if k == "name" {
             if let Value::Str(r, _) = v {
+                if r.is_empty() {
+                    p_name = None;
+                    continue;
+                }
                 let mut pb = PathBuf::new();
                 let s = Path::new(OsStr::from_bytes(r));
                 if !s.is_absolute() {
@@ -329,7 +345,7 @@ fn path_script_name(path: &Body, pid: u32, ppid: u32, cwd: &[u8], exe: &[u8]) ->
                         _ => tpb.push(c),
                     }
                 }
-                name = Some(NVec::from(tpb.as_os_str().as_bytes()))
+                p_name = Some(NVec::from(tpb.as_os_str().as_bytes()))
             }
         } else if k == "inode" {
             if let Value::Number(Number::Dec(i)) = v {
@@ -343,14 +359,15 @@ fn path_script_name(path: &Body, pid: u32, ppid: u32, cwd: &[u8], exe: &[u8]) ->
                     .collect::<Vec<_>>()
                     .try_into()
                     .ok()
-                    .map(|a: [u64; 2]| nix::sys::stat::makedev(a[0], a[1]));
+                    .map(|a: [u64; 2]| makedev(a[0], a[1]));
             }
             break;
         }
     }
-    match (p_dev, p_inode, name) {
-        (Some(p_dev), Some(p_inode), _) if p_dev == e_dev && p_inode == e_inode => None,
-        (Some(_), Some(_), Some(name)) if name != exe => Some(name),
+    match (p_dev, p_inode, p_name) {
+        (Some(p_dev), _, _) if major(p_dev) == 0 && major(e_dev) != 0 => None,
+        (Some(p_dev), Some(p_inode), _) if (p_dev, p_inode) == (e_dev, e_inode) => None,
+        (Some(_), Some(_), Some(p_name)) if p_name != exe => Some(p_name),
         _ => None,
     }
 }
@@ -575,17 +592,35 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
     }
 
     /// Enrich SOCKADDR record
-    fn enrich_sockaddr(&mut self, rv: &mut Body) {
+    ///
+    /// This function also determines whether the record should be filtered
+    fn enrich_sockaddr(&mut self, rv: &mut Body, is_filtered: &mut bool) {
         let mut nrv = Body::default();
         rv.retain(|(k, v)| match (k, v) {
-            (k, Value::Str(vr, _q)) if self.settings.translate_universal => {
+            (k, Value::Str(vr, _q)) => {
                 if k == "saddr" {
                     #[cfg(target_os = "linux")]
                     if let Ok(sa) = SocketAddr::parse(vr) {
-                        add_translated_socketaddr(&mut nrv, sa);
-                        return false;
+                        // There's no need to enrich saddr entries
+                        // that will be dropped, but the raw data
+                        // should be kept in case filter.filter-action
+                        // is set to "log".
+                        if *is_filtered {
+                            return true;
+                        } else if self.settings.filter_sockaddr.iter().any(|f| f.matches(&sa)) {
+                            *is_filtered = true;
+                            return true;
+                        }
+                        if self.settings.translate_universal {
+                            add_translated_socketaddr(&mut nrv, sa);
+                            return false;
+                        } else {
+                            return true;
+                        }
                     }
-                } else if k == "SADDR" {
+                } else if k == "SADDR" && self.settings.translate_universal || *is_filtered {
+                    // If we do our own enrichment, drop pre-existing
+                    // enriched SOCKADDR.saddr enrichment.
                     return false;
                 }
                 true
@@ -873,6 +908,12 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
             }
         }
 
+        if let Some(EventValues::Multi(ref mut rvs)) = ev.body.get_mut(&MessageType::SOCKADDR) {
+            for rv in rvs {
+                self.enrich_sockaddr(rv, &mut ev.is_filtered)
+            }
+        }
+
         if ev.is_filtered {
             return;
         }
@@ -885,9 +926,6 @@ impl<'a, 'ev> Coalesce<'a, 'ev> {
                     self.enrich_syscall(rv, ev.process_key, &script, &mut container_info)
                 }
                 (&MessageType::EXECVE, EventValues::Single(_)) => {}
-                (&MessageType::SOCKADDR, EventValues::Multi(rvs)) => {
-                    rvs.iter_mut().for_each(|rv| self.enrich_sockaddr(rv))
-                }
                 (&MessageType::PROCTITLE, EventValues::Single(rv)) => self.transform_proctitle(rv),
                 (&MessageType::URINGOP, EventValues::Multi(rvs)) => {
                     rvs.iter_mut().for_each(|rv| self.enrich_uringop(rv))
@@ -1831,6 +1869,29 @@ mod test {
         }
     }
 
+    #[test]
+    fn filter_sockaddr() {
+        for filter in &[
+            &["127.0.0.1", "::1"][..],
+            &["127.0.0.0/8", "::/64"],
+            &["127.0.0.1:11211", "[::1]:11211"],
+            &["127.0.0.0/8:11211", "[::/64]:11211"],
+            &["*:11211"],
+        ] {
+            {
+                let events: Rc<RefCell<Vec<Event>>> = Rc::new(RefCell::new(vec![]));
+                let mut c = Coalesce::new(mk_emit_vec(&events));
+                c.settings.filter_first_per_process = true;
+                c.settings.filter_sockaddr = filter.iter().map(|s| s.parse().unwrap()).collect();
+                process_record(&mut c, include_bytes!("testdata/record-connect.txt")).unwrap();
+                let events = events.borrow();
+                println!("{events:?}");
+                assert!(!events.iter().any(|e| &e.id == "1723819442.459:2482681"));
+                assert!(!events.iter().any(|e| &e.id == "1723819442.459:2482682"));
+            }
+        }
+    }
+
     fn gen_long_find_execve() -> Vec<u8> {
         let mut buf = vec![];
         let msgid = "1663143990.204:2148478";
@@ -2106,7 +2167,7 @@ type=EOE msg=audit(1740992884.191:7058722):
 
             let events = events.borrow();
             for event in events.iter() {
-                println!("{}", event_to_json(&event));
+                println!("{}", event_to_json(event));
             }
 
             let id = "1740992884.191:7058722";
