@@ -191,21 +191,21 @@ mod test {
     #[test]
     fn existing() {
         let td = mkdtemp(&temp_dir().join("laurel-test-XXXXXXXX")).expect("can't create temp dir");
-        std::fs::write(&td.join("logfile"), "asdf").expect("setup");
-        let mut fr = FileRotate::new(&td.join("logfile")).with_generations(3);
+        std::fs::write(td.join("logfile"), "asdf").expect("setup");
+        let mut fr = FileRotate::new(td.join("logfile")).with_generations(3);
         fr.rotate().expect("rotate");
         assert!(
-            std::fs::exists(&td.join("logfile.1")).expect("stat"),
+            td.join("logfile.1").exists(),
             "after rotate, logfile.1 should exist"
         );
         assert!(
-            !std::fs::exists(&td.join("logfile")).expect("stat"),
+            !td.join("logfile").exists(),
             "after rotate, logfile should no longer exist"
         );
         fr.write(b"asdf").expect("write");
         fr.flush().expect("flush");
         assert!(
-            std::fs::exists(&td.join("logfile")).expect("stat"),
+            td.join("logfile").exists(),
             "after rotate+write, logfile should exist"
         );
         std::fs::remove_dir_all(td).expect("remove_dir_all");
